<?php

namespace Drupal\ubercart_funds\Plugin\Ubercart\PaymentMethod;

use Drupal\Core\Form\FormStateInterface;
use Drupal\uc_order\OrderInterface;
use Drupal\uc_payment\PaymentMethodPluginBase;
use Drupal\Core\Url;

/**
 * Defines a funds account payment method.
 *
 * Allows user to pay with their balance account.
 *
 * @UbercartPaymentMethod(
 *   id = "funds",
 *   name = @Translation("Funds account")
 * )
 */
class FundsPayment extends PaymentMethodPluginBase {

  /**
   * {@inheritdoc}
   */
  public function cartDetails(OrderInterface $order, array $form, FormStateInterface $form_state) {
    return array(
      '#markup' => $this->t('Funds - Pay with your balance account.'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function orderSubmit(OrderInterface $order) {
    $total = $order->getTotal();
    $account = \Drupal::currentUser();
    $connexion = \Drupal::database();
    $query = $connexion->query('SELECT * FROM uc_funds_user_funds WHERE uid = :uid', [
      ':uid' => $account->id(),
    ]);
    $funds = $query->fetchObject();
    $user_balance = $funds ? $funds->balance : 0;

    if ($total > ($user_balance / 100)) {
      $message = $this->t('You don\'t have enough funds to cover this payment. Please <a href="@deposit_link">deposit funds</a> first.', [
        '@deposit_link' => Url::fromRoute('uc_funds.deposit')->toString(),
        ]
      );
      return $message;
    }

    // Update user balance in db.
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => $account->id()])
      ->fields([
        'uid' => $account->id(),
        'balance' => $user_balance -= ($total * 100),
      ])
      ->execute();

    uc_payment_enter($order->id(), 'funds', 0, 0, NULL, $this->t('Checkout completed with account funds.'));
  }

}
